import json
import time
import uuid
from urllib.parse import quote_plus

from pb_common.utils import support
from pb_common.utils.logger import get_logger
from pb_common.client.exceptions import ClientRegistryError, AuthTokenException

__author__ = "Maksim Senchuk"

logger = get_logger(__name__)


class ClientRegistration(object):
    """
    Регистрация в ДБО
    """

    def __init__(self, context):
        self.token_provider = context.token_provider_service

    def create(self, integration_id, client_type, phone, password, has_password, login, has_login, name):
        """
        Зарегистрировать клиента в ДБО через TokenProvider
        :param integration_id: интегрейшн айди клиента
        :param client_type: тип клиента Full/Lite/LiteIdent
        :param phone: номер телефона клиента
        :param password: пароль
        :param has_password: признак наличия пароля True/False
        :param login: логин
        :param has_login: признак наличия логина True/False
        :param name: имя пользователя: Иванов Иван Иванович
        """
        session_info = {
            "integrationId": integration_id,
            "attributes": {
                "cn": login,
                "userPassword": password,
                "uid": str(uuid.uuid4()),
                "mobile": phone,
                "clientDisabled": "false",
                "clientType": client_type,
                "hasLogin": str(has_login).lower()
            }
        }
        if has_password is not None:
            session_info['attributes'].update(dict(hasPassword=str(has_password).lower()))
        if name:
            session_info['attributes'].update(dict(displayName=name))
        body = """client_id=factura&client_secret=ekassir&grant_type=sign_up&session_info={}""".format(
            quote_plus(json.dumps(session_info, ensure_ascii=False)))
        r = self.token_provider.auth_token(data=body)
        if r.status_code == 200:
            return r.json()
        raise ClientRegistryError('Клиент {} не зарегистрирован в ДБО'.format(integration_id))

    def delete(self, integration_id):
        """
        Удалить регистрацию клиента в ДБО через TokenProvider
        :param integration_id: интегрейшн айди клиента
        """
        session_info = {"integration_id": integration_id}
        body = """client_id=factura&client_secret=ekassir&grant_type=sign_out&session_info={}""".format(
            quote_plus(json.dumps(session_info, ensure_ascii=False)))
        r = self.token_provider.auth_token(data=body)
        logger.debug(r.text)

    @support.retry_request(delay=5.0, exceptions=(AuthTokenException,))
    def get_token(self, integration_id):
        """Сгенерировать новый токен"""
        current_time = time.time()
        r = self.token_provider.auth_token(
            data=f'grant_type=assertion&client_id=factura&client_secret='
                 f'ekassir&assertion_type=siebelID&assertion={integration_id}')
        if r.status_code != 200:
            raise AuthTokenException(r.text)
        data = r.json()
        data['expires_in'] = current_time + data['expires_in']
        return {
            'token': f'{data["token_type"]} {data["access_token"]}',
            'expired': data['expires_in']
        }
